require("defines")
-- debug_status = 1
debug_file = "score-deb.txt"
debug_mod_name = "ScoreExtended"
require("utils")
require("config")

--------------------------------------------------------------------------------------
local function init_force(force)
	-- initialize or update per force globals of the mod
	debug_print( "init_force ", force.name )
	
	global.force_mem[force.name] = global.force_mem[force.name] or {}
	local force_mem = global.force_mem[force.name]
	
	force_mem.ticks_erase = force_mem.ticks_erase or 0
	
	force_mem.rockets_tot = force_mem.rockets_tot or 0
	force_mem.rockets_count = force_mem.rockets_count or 0
	force_mem.rockets_count_ticks = force_mem.rockets_count_ticks or 0
	force_mem.rockets_average = force_mem.rockets_average or 0
	force_mem.rockets_average2 = force_mem.rockets_average2 or 0
	
	force_mem.silos = force_mem.silos or {}
	if force_mem.autolaunch == nil then force_mem.autolaunch = autolauch_default end
end

--------------------------------------------------------------------------------------
local function init_forces()
	local force
	
	for _, force in pairs(game.forces) do
		init_force(force)
	end

	if global.no_silos then
		find_silos()
		global.no_silos = false
	end
end

--------------------------------------------------------------------------------------
local function init_player(player)
	-- initialize or update per player globals of the mod, and reset the gui
	debug_print( "init_player ", player.name, " connected=", player.connected )
	
	global.player_mem[player.index] = global.player_mem[player.index] or {}
	
	local player_mem = global.player_mem[player.index]
	
	if player_mem.gui_opened == nil then player_mem.gui_opened = false end
	if player_mem.autoshow == nil then player_mem.autoshow = true end
		
	if player.connected then
		build_bar(player)
	end
end

--------------------------------------------------------------------------------------
local function init_players()
	local player
	for _, player in pairs(game.players) do
		init_player(player)
	end
end

--------------------------------------------------------------------------------------
local function init_globals()
	-- initialize or update general globals of the mod
	global.ticks = global.ticks or 0
	global.force_mem = global.force_mem or {}
	global.player_mem = global.player_mem or {}

	if global.no_silos == nill then global.no_silos = true end
end

--------------------------------------------------------------------------------------
script.on_init( 
	function()
		debug_print( "on_init" )
		init_globals()
		init_forces()
	end
)

--------------------------------------------------------------------------------------
local function on_configuration_changed(data)
	-- detect any mod or game version change
	if data.mod_changes ~= nil and data.mod_changes[debug_mod_name] ~= nil then
		debug_print( "update mod: ", debug_mod_name )
		
		init_globals()
	
		init_forces()
		
		init_players()
	end
end

script.on_configuration_changed(on_configuration_changed)

--------------------------------------------------------------------------------------

local function on_force_created(event)
	local force = event.force
	debug_print( "force created ", force.name )
	
	init_force(force)
end
	
script.on_event(defines.events.on_force_created, on_force_created)

--------------------------------------------------------------------------------------
function on_player_created(event)
	local player = game.get_player(event.player_index)
	debug_print( "player created ", player.name )
	
	init_player(player)
end

script.on_event(defines.events.on_player_created, on_player_created )

--------------------------------------------------------------------------------------
function on_creation( event )
	local ent = event.created_entity
	local force_mem = global.force_mem[ent.force.name]
	
	if ent.type == "rocket-silo" then
		debug_print( "creation ", ent.name )
		
		table.insert(force_mem.silos,ent)
	end
end

script.on_event(defines.events.on_built_entity, on_creation )
script.on_event(defines.events.on_robot_built_entity, on_creation )

--------------------------------------------------------------------------------------
function on_destruction( event )
	local ent = event.entity
	local force_mem = global.force_mem[ent.force.name]
	
	if ent.type == "rocket-silo" then
		debug_print( "destruction ", ent.name )
		del_list(force_mem.silos,ent)
	end
end

script.on_event(defines.events.on_entity_died, on_destruction )
script.on_event(defines.events.on_robot_pre_mined, on_destruction )
script.on_event(defines.events.on_preplayer_mined_item, on_destruction )

--------------------------------------------------------------------------------------
local function on_tick(event)
	if global.ticks <=0 then
		global.ticks = 91	
		
		local force, force_mem, player, player_mem
		local disable_on_tick = true
		
		-- hide score window after delay, if not opened manually before
		for _, force in pairs(game.forces) do
			force_mem = global.force_mem[force.name]
			
			if force_mem.ticks_erase ~= 0 and game.tick >= force_mem.ticks_erase then
				for _, player in pairs(force.players) do
					if player.connected then
						player_mem = global.player_mem[player.index]
						if player_mem.autoshow and (not player_mem.gui_opened) and player.gui.left.frm_score ~= nil then
							player.gui.left.frm_score.destroy()
						end
					end
				end
				update_bars( force, false )
				force_mem.ticks_erase = 0
			else
				disable_on_tick = false
			end
		end
		
		-- check autolaunch
		
		local k, silo

		for _, force in pairs(game.forces) do
			force_mem = global.force_mem[force.name]
			
			if force_mem.autolaunch then
				for k, silo in pairs( force_mem.silos ) do
					if silo.valid then
						if silo.get_item_count("satellite") > 0 then
							silo.launch_rocket()
							update_bars( force, true )
						end
						-- local ret, invent = pcall(silo.get_inventory, 5)
						-- if ret and not invent.is_empty() then
							-- silo.launch_rocket()
						-- end
					else
						table.remove(force_mem.silos,k)
					end
				end
			end
		end
	else
		global.ticks = global.ticks - 1
	end
end

script.on_event(defines.events.on_tick, on_tick)

--------------------------------------------------------------------------------------

local function on_rocket_launched(event)
	local force = event.rocket.force
	
	local force_mem = global.force_mem[force.name]
	local previous_count = false
	
	-- destroy vanilla score window and try to get rocket count if not known yet...
	for _, player in pairs(force.players) do
		if player.connected then
			if destroy_vanilla(player, force_mem) then previous_count = true end
		end
	end

	-- update stats and display window
	
	force_mem.ticks_erase = game.tick + score_delay * 60
	if not previous_count then 
		force_mem.rockets_tot = force_mem.rockets_tot + 1
		force_mem.rockets_count = force_mem.rockets_count + 1
	end
	local dt = game.tick - force_mem.rockets_count_ticks
	if dt == 0 or force_mem.rockets_count == 0 then
		force_mem.rockets_average = 0
		force_mem.rockets_average2 = 0
	else
		force_mem.rockets_average = math.floor(force_mem.rockets_count / (dt / 60) * 60 * 60 * 10) / 10 -- calculate average per hour
		force_mem.rockets_average2 = math.floor(10 / (force_mem.rockets_count / (dt / 60) * 60)) / 10  -- calculate period in minutes
	end
	
	if score_delay >= 0 then
		local player_mem
		for _, player in pairs(force.players) do
			if player.connected then
				player_mem = global.player_mem[player.index]
				if player_mem.autoshow then
					update_gui(player, force_mem)
				end
				player.gui.top.but_score_main.style = "score_button_rocket_on_style"
			end
		end
	end
end

script.on_event(defines.events.on_rocket_launched, on_rocket_launched)

--------------------------------------------------------------------------------------
function on_gui_click(event)
	local player = game.get_player(event.player_index)
	local event_name = event.element.name
	
	-- debug_print( "player ", player.name, " click ", event_name )

	if event_name == "but_score_main" then
		local player_mem = global.player_mem[player.index]
		
		if player.gui.left.frm_score == nil then
			update_gui(player)
			player_mem.gui_opened = true
		else
			player.gui.left.frm_score.destroy()
			player_mem.gui_opened = false
		end
		
	elseif event_name == "chk_score_autolaunch" then
		local force_mem = global.force_mem[player.force.name]
		force_mem.autolaunch = not force_mem.autolaunch
		update_guis(player.force)
		
	elseif event_name == "chk_score_autoshow" then
		local player_mem = global.player_mem[player.index]
		player_mem.autoshow = not player_mem.autoshow
		update_guis(player.force)
		
	elseif event_name == "but_score_count_rst" then
		local force_mem = global.force_mem[player.force.name]
		force_mem.rockets_count = 0
		force_mem.rockets_count_ticks = game.tick
		force_mem.rockets_average = 0
		force_mem.rockets_average2 = 0
		update_guis(player.force)
	end
end

script.on_event(defines.events.on_gui_click, on_gui_click)

--------------------------------------------------------------------------------------
local function clean_gui(gui)
	local guiname
	for _, guiname in pairs( gui.children_names ) do
		gui[guiname].destroy()
	end
end

--------------------------------------------------------------------------------------
function build_bar( player )
	-- debug_print("update bar player" .. player.name)
	
	if player.gui.top.but_score_main == nil then
		player.gui.top.add({type = "button", name = "but_score_main", style = "score_button_rocket_off_style"})
	end
end

--------------------------------------------------------------------------------------
function update_bars( force, launching )
	local player
	for _, player in pairs(force.players) do
		if player.connected then
			if launching then
				player.gui.top.but_score_main.style = "score_button_rocket_on_style"
			else
				player.gui.top.but_score_main.style = "score_button_rocket_off_style"
			end
		end
	end
end

--------------------------------------------------------------------------------------
function update_gui( player, force_mem )
	-- debug_print("update gui player" .. player.name)
	
	if force_mem == nil then
		force_mem = global.force_mem[player.force.name]
	end
	
	destroy_vanilla(player,force_mem)

	local gui1, gui2
	
	if player.gui.left.frm_score == nil then
		gui1 = player.gui.left.add({type = "frame", name = "frm_score", caption = {"score-gui-title"}, direction = "vertical", style = "score_frame_style"})
	else
		gui1 = player.gui.left.frm_score
		clean_gui(gui1)
	end

	local player_mem = global.player_mem[player.index]

	gui1.add({type = "label", name = "lbl_score_tot", caption = {"score-gui-tot",force_mem.rockets_tot}, style = "score_label_style"})				
	gui2 = gui1.add({type = "frame", name = "frm_score_count", direction = "vertical", style = "score_frame_style"})
	gui2.add({type = "label", name = "lbl_score_nb", caption = {"score-gui-nb",force_mem.rockets_count}, style = "score_label_style"})				
	gui2.add({type = "label", name = "lbl_score_av1", caption = {"score-gui-av1",force_mem.rockets_average}, style = "score_label_style"})				
	gui2.add({type = "label", name = "lbl_score_av2", caption = {"score-gui-av2",force_mem.rockets_average2}, style = "score_label_style"})				
	gui2.add({type = "button", name = "but_score_count_rst", caption = {"score-gui-reset"}, style = "score_button_style"})
	gui1.add({type = "checkbox", name = "chk_score_autolaunch", caption = {"score-gui-autolaunch"}, state = force_mem.autolaunch, style = "score_checkbox_style"})				
	gui1.add({type = "checkbox", name = "chk_score_autoshow", caption = {"score-gui-autoshow"}, state = player_mem.autoshow, style = "score_checkbox_style"})				
end
	
--------------------------------------------------------------------------------------
function update_guis( force )
	local force_mem = global.force_mem[force.name]
	local player
	
	for _, player in pairs(force.players) do
		if player.connected then
			update_gui(player, force_mem)
		end
	end
end

--------------------------------------------------------------------------------------
function destroy_vanilla( player, force_mem )
	if player.gui.left.rocket_score == nil then return( false ) end
	
	local previous_count = false
	
	if force_mem.rockets_tot == 0 then -- retrieve previous rocket total if no stat recorded yet (in case of late mod install)
		local tot = tonumber(player.gui.left.rocket_score.rocket_count.caption)
		if tot == nil then
			force_mem.rockets_tot = 0
			force_mem.rockets_count = 0
		else
			force_mem.rockets_tot = tot
			force_mem.rockets_count = tot
			local dt = game.tick - force_mem.rockets_count_ticks
			if dt == 0 or force_mem.rockets_count == 0 then
				force_mem.rockets_average = 0
				force_mem.rockets_average2 = 0
			else
				force_mem.rockets_average = math.floor(force_mem.rockets_count / (dt / 60) * 60 * 60 * 10) / 10 -- calculate average per hour
				force_mem.rockets_average2 = math.floor(10 / (force_mem.rockets_count / (dt / 60) * 60)) / 10  -- calculate period in minutes
			end
			previous_count = true
		end
	end
	
	player.gui.left.rocket_score.destroy()
	
	return(previous_count)	
end
	
--------------------------------------------------------------------------------------
function find_silos()
	-- search for existing silos (if late mod install)
	local surf , chunk, silo, force, force_mem
	
	for _, force in pairs(game.forces) do
		force_mem = global.force_mem[force.name]
		force_mem.silos = {}
	end	
		
	for _, surf in pairs(game.surfaces) do
		for chunk in surf.get_chunks() do
			a = {{chunk.x * 32, chunk.y * 32}, {chunk.x * 32 + 32, chunk.y * 32 + 32}}
			for _, silo in pairs(surf.find_entities_filtered({area=a, type= "rocket-silo"})) do
				force_mem = global.force_mem[silo.force.name]
				-- table.insert(force_mem.silos,silo)
				add_list(force_mem.silos,silo) -- to avoid duplicates (1 silo can be on 2 chunks)
			end
		end
	end
	
	for _, force in pairs(game.forces) do
		force_mem = global.force_mem[force.name]
		debug_print( "force ", force.name, " silos ", #force_mem.silos )
	end
	
	return(nil)
end

--------------------------------------------------------------------------------------
function add_list(list, obj)
	-- to avoid duplicates...
	local obj2, i
	for i, obj2 in pairs(list) do
		if obj2 == obj then
			return(false)
		end
	end
	table.insert(list,obj)
	return(false)
end

--------------------------------------------------------------------------------------
function del_list(list, obj)
	local obj2, i
	for i, obj2 in pairs(list) do
		if obj2 == obj then
			table.remove( list, i )
			return(true)
		end
	end
	return(false)
end

